
/* --- C ---
************************************************************************
*
*   Filename    : mpegtc2.c
*   Description : corrects or shows the GOP time code of a MPEG-1-Video-stream
*   Part of     : MPEGTC2
*
*   Version     : 1.0
*   Date        : Wed Oct 13 21:34:14 MDT 1993
*   Language    : C
*   For machine : Linux, Sun OS, SCO, AU/X, Solaris
*   Compile as  : see Makefile
*
*   Author      : Frank Gadegast
*   Contact     : phade@cs.tu-berlin.de
*   Copyright   : PHADE Software, Berlin
*
*   spd@daphne.cps.unizar.es ftime -> time 
*
************************************************************************
*/

#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#ifdef DOS
#define INPUT_ACCESS  O_BINARY|O_RDONLY
#define OUTPUT_ACCESS O_BINARY|O_CREAT|O_TRUNC|O_RDWR
#else
#define INPUT_ACCESS  O_RDONLY
#define OUTPUT_ACCESS O_CREAT|O_TRUNC|O_RDWR
#endif
#define OUTPUT_PERMS  S_IWRITE|S_IREAD

#define BUFSIZE 20000

char buf[BUFSIZE];

#include <sys/timeb.h>

void share_wait ()
{
  time_t t1,t2;
  int j;

  time(&t1);
  do
  {
    for (j=0; j<500; j++)
    time(&t2);
    fprintf (stderr, ".");
  }
  while ((t2-t1) < 5);
  fprintf (stderr, "\n");
}

int main(argc, argv)
int argc;
char *argv[];
{
	int f, s, n;
	int c, i, j, sk=0;
	int arg;
	int frames = 0;
	int mode = 0;

	long all, nl;
	unsigned int drop, hh, mm, marker, ss, ff, gop, link;
	unsigned int nhh, nmm, nss, nff, fps;

#ifdef SHARE
	fprintf (stderr, "\nMPEGTC Version 1.1, 08/02/96, MPEG-1-Video TimeCode changer\n");
	fprintf (stderr, "       Copyright (C) 1995-96 by PHADE Software\n\n");

	fprintf(stderr, "Non-registered shareware-version. Please read the\n");
	fprintf(stderr, "README.TXT-file to support shareware.\n\n");
#else
	printf ("\nMPEGTC Version 1.1, 08/02/96, MPEG-1-Video TimeCode changer\n");
	printf ("       Copyright (C) 1995-96 by PHADE Software\n\n");
#endif


	if (argc < 2)
	{
		fprintf (stderr, "usage: %s source.mpg [target.mpg fps hh mm ss ff]\n", argv[0]);
		exit (-1);
	}
	if (argc == 2) mode = 1;
	else
	{
		fps = atoi (argv[3]);
		nhh = atoi (argv[4]);
		nmm = atoi (argv[5]);
		nss = atoi (argv[6]);
		nff = atoi (argv[7]);
	}

	s = open (argv[1], INPUT_ACCESS);
	if (s < 1)
	{
		fprintf (stderr, "%s: can't open input stream '%s'\n", argv[0], argv[arg]);
		exit (-1);
	}
#ifdef SHARE
	fprintf (stderr, "HH:MM:SS.FF d m g l ");
#else
	printf ("HH:MM:SS.FF d m g l ");
#endif
	if (!mode)
	{
		n = open (argv[2], OUTPUT_ACCESS, OUTPUT_PERMS);
		if (n < 1)
		{
			fprintf (stderr, "%s: can't open output stream '%s'\n", argv[0], argv[3]);
			exit (-1);
		}
#ifdef SHARE
		fprintf (stderr, "HH:MM:SS.FF");
#else
		printf ("HH:MM:SS.FF");
#endif
	}
#ifdef SHARE
	fprintf (stderr, "\n");
	fprintf (stderr, "--------------------");
	if (!mode) fprintf (stderr, "-----------");
	fprintf (stderr, "\n");
#else
	printf ("\n");
	printf ("--------------------");
	if (!mode) printf ("-----------");
	printf ("\n");
#endif

	do
	{
		c = read (s, buf, BUFSIZE);
		for (i=0; i<c; i++)
			{
				if ((buf[i] == 0) &&
					(buf[i+1] == 0) &&
					(buf[i+2] == 1) &&
					(buf[i+3] == 0))
				{
					frames++;
					nff++;
					if (nff > fps-1)
					{
						nff = 0;
						nss++;
						if (nss > 59)
						{
							nss = 0;
							nmm++;
							if (nmm > 59)
							{
								nmm = 0;
								nhh++;
							}
						}
					}
				}
				if ((buf[i] == 0) &&
					(buf[i+1] == 0) &&
					(buf[i+2] == 1) &&
					(buf[i+3] == -72))
				{
					all = (unsigned char)buf[i+4]; all = all << 8;
					all += (unsigned char)buf[i+5]; all = all << 8;
					all += (unsigned char)buf[i+6]; all = all << 8;
					all += (unsigned char)buf[i+7];
					ff = all << 19; ff = ff >> 26;
					ss = all << 13; ss = ss >> 26;
					mm = all << 6; mm = mm >> 26;
					hh = all << 1; hh = hh >> 26;
					drop = all; drop = drop >> 31;
					marker = all << 12; marker = marker >> 31;
					gop = all << 25; gop = gop >> 31;
					link = all << 26; link = link >> 31;
#ifdef SHARE
					fprintf (stderr, "%02d:%02d:%02d.%02d %01x %01x %01x %01x ",
						hh, mm, ss, ff,
						drop, marker, gop, link);
#else
					printf ("%02d:%02d:%02d.%02d %01x %01x %01x %01x ",
						hh, mm, ss, ff,
						drop, marker, gop, link);
#endif
					if (!mode)
					{
						all = 0;
						all += drop; all = all << 5;
						all += nhh; all = all << 6;
						all += nmm; all = all << 1;
						all += marker; all = all << 6;
						all += nss; all = all << 6;
						all += nff; all = all << 1;
						all += gop; all = all << 1;
						all += link; all = all << 5;
						nl = all >> 24;
						buf[i+4] = (unsigned int)nl;
						nl = all << 8; nl = nl >> 24;
						buf[i+5] = (unsigned int)nl;
						nl = all << 16; nl = nl >> 24;
						buf[i+6] = (unsigned int)nl;
						nl = all << 24; nl = nl >> 24;
						buf[i+7] = (unsigned int)nl;
#ifdef SHARE
						fprintf (stderr, "%02d:%02d:%02d.%02d ", nhh, nmm, nss, nff);
#else
						printf ("%02d:%02d:%02d.%02d\n", nhh, nmm, nss, nff);
#endif
					}
#ifdef SHARE
	share_wait ();
#else
					else printf ("\n");
#endif
				}
			}
		if (!mode) write (n, buf, c); 
	} while (c == BUFSIZE);
#ifdef SHARE
	fprintf (stderr, "--------------------");
	if (!mode) fprintf (stderr, "-----------");
	fprintf (stderr, "\n");
	fprintf (stderr, "frames: %d\n", frames);
#else
	printf ("--------------------");
	if (!mode) printf ("-----------");
	printf ("\n");
	printf ("frames: %d\n", frames);
#endif
	close (s);
	if (!mode) close (n);

	exit (0);
}

